<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Login extends Controller_Template {

	public $template = 'template/login_tpl';

	public function action_index()
	{
		$session = Session::instance();

		$session->delete('tmp_user_id');
		$session->delete('tmp_user_name');
		$session->delete('tmp_user_role_id');

		// check if there's a valid user session
		if ($session->get('user_id'))
		{
			$role_id = $session->get('user_role_id');

			if ($role_id == 1 OR $role_id == 2)
			{
				$this->redirect('admin/users');
			}
			else if ($role_id == 3)
			{
				$this->redirect('admin/category');
			}
			else
			{
				$this->redirect('admin/cms');
			}
		}

		$msg = $session->get_once('msg');

		$this->template->navmenu = '';
		$this->template->content = View::factory('admin/login/index');
		$this->template->content->msg = $msg;
	}

	public function action_check()
	{
		$this->auto_render = FALSE;

		$session = Session::instance();

		$return = FALSE;
		$redirect = NULL;

		$int_login_try = Kohana::$config->load('site')->get('login_try');
		$sess_idle_time = Kohana::$config->load('site')->get('sess_idle_time');
		$sess_logout_time = Kohana::$config->load('site')->get('sess_logout_time');
		$int_max_login_try = Kohana::$config->load('site')->get('max_login_try');
		$int_inactive = Kohana::$config->load('site')->get('inactive');

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$username = HTML::chars(Arr::get($this->request->post(), 'username'));
				$passw = HTML::chars(Arr::get($this->request->post(), 'password'));
				// $password = crypt($passw, Cookie::$salt);

				// Test Log
				// MyHelper::login_log(array('uname'=>$username, 'pword'=>$passw));

				// Check if username exists in CMS db
				$check = ORM::factory('Admin')
				    ->where('username', '=', $username)
				    ->where('active', '=', 'Y')
				    ->find();

				if ($check->id)
				{
					$user_id = $check->id;
					$locked = $check->locked;
					$login_try = $check->login_try;
					$role_id = $check->role_id;
					$last_visit = $check->last_visit;
					$max_login_try = $check->max_login_try;
					$suspended = $check->suspended;

					if ($role_id == 8 OR $role_id == 9)
					{
						// do not allow user/data admin checker
						$msg = 'User account login is not allowed on this screen';
					}
					else
					{
						// check if this user is inactive for the past $int_inactive days
						if (Activity::check_inactive($user_id, $int_inactive))
						{
							$sql = DB::update('users')->set(array('suspended'=>'Y'))->where('id', '=', $user_id)->execute();
							$suspended = 'Y';
						}

						// all other user role
						if ($locked == 'Y' OR ($login_try >= $int_login_try) OR ($max_login_try >= $int_max_login_try) OR $suspended == 'Y')
						{
							if ($max_login_try >= $int_max_login_try)
							{
								$msg = "Your password has been locked due to ".$int_max_login_try." historical invalid login attempts";
								$log_description = "Account is locked due to ".$int_max_login_try." historical invalid login attempts";
							}
							else if ($login_try >= $int_login_try)
							{
								$msg = 'Your password has been locked due to '.$int_login_try.' invalid login attempts';
								$log_description = "Account is locked due to ".$int_login_try." consecutive invalid login attempts";
							}
							else if ($suspended == 'Y')
							{
								$msg = "Your account has been suspended due to non-usage over the specified period of time.";
								$log_description = "Account is locked due to inactivity for ".$int_inactive." days";
							}
							else if ($locked == 'Y')
							{
								$msg = "Your password has been locked due to invalid login attempts";
								$log_description = "Account is locked due to invalid login attempts";
							}

							if ($role_id <= 3 OR $role_id > 7)
							{
								$msg .= '<br />Please contact Level 1 User to re-activate your '.($suspended == 'Y' ? 'account' : 'password');
							} else {
								$msg .= '<br />Please contact the user administrator to re-activate your '.($suspended == 'Y' ? 'account' : 'password');
							}

							// For Activity
							$log_action = "Log In";
							Activity::add($user_id, $log_action, $log_description);
							$return = FALSE;
						}
						else
						{
							$user = ORM::factory('Admin')
								->where('username', '=', $username)
								// ->and_where('password', '=', $password)
								->and_where('active', '=', 'Y')
								->find();

							if ($user->id)
							{
								// User Admin and Data Admin
								if ($role_id == 2 OR $role_id == 3)
								{
									$session->set('tmp_user_id', $user->id);
									$session->set('tmp_user_name', $user->name);
									$session->set('tmp_user_role_id', $user->role_id);
								}
								else
								{
									$session->set('user_id', $user->id);
									$session->set('user_name', $user->name);
									$session->set('user_role_id', $user->role_id);

									if ($role_id >= 4 AND $role_id <= 7)
									{
										// Maker, Checker, Checker 2, Publisher
										$session->set('user_category_ids', $check->category_ids);
										$session->set('user_segment_ids', $check->segment_ids);

										$redirect = 'cms';
									}
									else if ($role_id == 1)
									{
										// Level 1 Admin
										$redirect = 'users';
									}
									else if ($role_id == 10)
									{
										// CMS Admin Monitor
										$redirect = 'logs';
									}

									// For concurrent user login check
									// check first if there's already a record in the sessions table for this user
									$qry = DB::select()
										->from('sessions')
										->where('user_id', '=', $user->id)
										->where('bootout', 'IS', NULL)
										->as_object()
										->execute();
									// echo Database::instance()->last_query.' test ';
									// exit;
									if ($qry->count())
									{
										$sql = $qry->current();
										$sess_id = $sql->session_id;
										DB::update('sessions')->set(array('bootout'=>'Y'))->where('session_id', '=', $sess_id)->execute();
									}

									// Save user id in session table for the new login
									$sql = DB::update('sessions')->set(array('user_id'=>$user->id))->where('session_id', '=', $session->id())->execute();
									// END concurrent user login check

									// update last login tag to today (maker, checker, checker2, publisher, l1 admin)
									$user->last_visit = date('Y-m-d H:i:s', time());
									$user->save();

									// reset login_try error count log to 0;
									$this->clear_account($user->id);
								}

								// set session idle and logout timer
								$session->set('sess_idle_time', $sess_idle_time);
								$session->set('sess_logout_time', $sess_logout_time);
								$session->set('last_login', $last_visit);

								// Log activity for all other user type except data admin and user admin
								// Both user needs verfication from the checker
								if ($role_id != 2 AND $role_id != 3)
								{
									// For Activity
									$log_action = "Log In";
									$log_description = "Logged in the system on ".date('Y-m-d H:i:s', time());

									// Log Activity
									Activity::add($check->id, $log_action, $log_description);
								}

								$return = TRUE;
								$msg = 'Login Sucessful';
							}
							else
							{
								// correct username but wrong password
								$count = $login_try + 1;
								$max_count = $max_login_try + 1;
								$sql = ORM::factory('Admin', $user_id);
								$role_id = $sql->role_id;
								$sql->login_try = $count;
								$sql->max_login_try = $max_count;
								$sql->save();

								// 3 consecutive wrong password attempt.  lock account
								$wrong_pass_count = $session->get('wrong_pass_count') ? $session->get('wrong_pass_count') : 0;

								if ($max_count >= $int_max_login_try)
								{
									$this->lock_account($user_id);

									$msg = "Your password has been locked due to ".$int_max_login_try." historical invalid login attempts";
									if ($role_id <= 3)
									{
										$msg .= '<br />Please contact Level 1 User to re-activate your password';
									} else {
										$msg .= '<br />Please contact the User Administrator to re-activate your password';
									}

									// For Activity
									$log_action = "Log In";
									$log_description = "Account is locked due to ".$int_max_login_try." accumulative invalid login attempts";

									// Log Activity
									Activity::add($user_id, $log_action, $log_description);
								}
								else if (($wrong_pass_count + 1) >= 3)
								{
									$this->lock_account($user_id);

									$msg = 'Your password has been locked due to '.$int_login_try.' invalid login attempts';
									if ($role_id <= 3)
									{
										$msg .= '<br />Please contact Level 1 User to re-activate your password';
									} else {
										$msg .= '<br />Please contact the User Administrator to re-activate your password';
									}

									// For Activity
									$log_action = "Log In";
									$log_description = "Account is locked due to ".$int_login_try." invalid login attempts";

									// Log Activity
									Activity::add($user_id, $log_action, $log_description);
								}
								else
								{
									$c = $wrong_pass_count + 1;
									$session->set('wrong_pass_count', $c);
									$session->set('login_err', 'Y');

									$msg = 'Username and Password do not match.  Please try again.';

									// For Activity
									$log_action = "Log In";
									$log_description = "Username and Password do not match. (Username exists but did not match the password)";

									// Log Activity
									Activity::add($user_id, $log_action, $log_description);
								}

								Kohana::$log->add(Log::ERROR, 'Unable to find username/password match, username found in our DB');

								$return = FALSE;
							}
						}
					}
				} else {
					$msg = 'Username and Password do not match.  Please try again';
					$return = FALSE;

					Kohana::$log->add(Log::ERROR, 'Username not found in our database');
				}
			}
			else
			{
				$msg = 'Possible CSRF issue detected';
				Kohana::$log->add(Log::ERROR, 'CSRF detected');
			}
		}
		else
		{
			$msg = 'Form was not submitted through POST';
			Kohana::$log->add(Log::ERROR, 'Form method is not POST');
		}

		if ($return == TRUE)
		{
			if ($user->role_id == 2 OR $user->role_id == 3)
			{
				// redirect to user admin or data admin checker
				$this->redirect('admin/login/confirm');
			}
			else
			{
				$this->redirect('admin/'.$redirect);
			}
		}
		else
		{
			$session->set('msg', $msg);
			$this->redirect('admin');
		}
	}

	private function lock_account($user_id)
	{
		Session::instance()->delete('wrong_pass_count');
		Session::instance()->delete('login_err');

		$sql = ORM::factory('Admin', $user_id);
		$sql->locked = 'Y';
		$sql->save();
	}

	private function clear_account($user_id)
	{
		Session::instance()->delete('wrong_pass_count');
		Session::instance()->delete('login_err');

		$sql = ORM::factory('Admin', $user_id);
		$sql->locked = '';
		$sql->login_try = 0;
		$sql->save();
	}

	/**
	 * User Admin and Data Admin Checker Login Form
	 * @return [type] [description]
	 */
	public function action_confirm()
	{
		$session = Session::instance();
		$role_id = $session->get('tmp_user_role_id');

		$msg = $session->get_once('msg');

		$this->template->navmenu = '';
		$this->template->content = View::factory('admin/login/checker');
		$this->template->content->user_role = $role_id == 2 ? 'User Admin Checker' : ($role_id == 3 ? 'Data Admin Checker' : '');
		$this->template->content->msg = $msg;
	}

	/**
	 * User Admin and Data Admin Checker Login Check
	 * @return [type] [description]
	 */
	public function action_confirm_check()
	{
		$this->auto_render = FALSE;

		$session = Session::instance();

		$return = FALSE;
		$redirect = NULL;
		$msg = 'Username and Password do not match.  Please try again.';
		$int_login_try = Kohana::$config->load('site')->get('login_try');
		$sess_idle_time = Kohana::$config->load('site')->get('sess_idle_time');
		$sess_logout_time = Kohana::$config->load('site')->get('sess_logout_time');
		$int_max_login_try = Kohana::$config->load('site')->get('max_login_try');
		$int_inactive = Kohana::$config->load('site')->get('inactive');

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$username = HTML::chars(Arr::get($this->request->post(), 'username'));
				$passw = HTML::chars(Arr::get($this->request->post(), 'password'));
				$password = crypt($passw, Cookie::$salt);

				$checker_role_id = $session->get('tmp_user_role_id') == 2 ? 8 : 9;

				$check = ORM::factory('Admin')
				    ->where('username', '=', $username)
				    ->where('active', '=', 'Y')
				    ->where('role_id', '=', $checker_role_id)
				    ->find();

				if ($check->id)
				{
					$user_id = $check->id;
					$locked = $check->locked;
					$login_try = $check->login_try;
					$role_id = $check->role_id;
					$suspended = $check->suspended;

					// check if this user is inactive for the past $int_inactive days
					if (Activity::check_inactive($user_id, $int_inactive))
					{
						$sql = DB::update('users')->set(array('suspended'=>'Y'))->where('id', '=', $user_id)->execute();
						$suspended = 'Y';
					}

					// checked if locked or login try exceeded the defined try count
					if ($locked == 'Y' OR $login_try >= $int_login_try OR $suspended == 'Y')
					{
						if ($login_try >= $int_login_try)
						{
							$msg = 'Your password has been locked due to '.$int_login_try.' invalid login attempts';
							$msg .= '<br />Please contact Level 1 User to re-activate your password';
							$log_description = "Account is locked due to ".$int_login_try." consecutive invalid login attempts";
						}
						else if ($suspended == 'Y')
						{
							$msg = "Your account has been suspended due to non-usage over the specified period of time.";
							$log_description = "Account is locked due to inactivity for ".$int_inactive." days";
						}
						else if ($locked == 'Y')
						{
							$msg = "Your password has been locked due to invalid login attempts";
							$msg .= '<br />Please contact Level 1 User to re-activate your password';
							$log_description = "Account is locked due to invalid login attempts";
						}

						$return = FALSE;
						// For Activity
						$log_action = "Log In";

						// Log Activity
						Activity::add($user_id, $log_action, $log_description);
					}
					else
					{
						$user = ORM::factory('Admin')
						    ->where('username', '=', $username)
						    // ->and_where('password', '=', $password)
						    ->and_where('active', '=', 'Y')
						    ->find();

						if ($user->id)
						{
							// make user admin and data admin from first login screen become active (deleting temporary account placeholder)
							$u_id = $session->get_once('tmp_user_id');
							$u_name = $session->get_once('tmp_user_name');
							$u_role_id = $session->get_once('tmp_user_role_id');
							$session->set('user_id', $u_id);
							$session->set('user_name', $u_name);
							$session->set('user_role_id', $u_role_id);

							$session->set('checker_id', $user->id);
							$session->set('checker_name', $user->name);

							$user->last_visit = date('Y-m-d H:i:s', time());
							$user->save();

							// reset login_try error count log to 0;
							$this->clear_account($user->id);

							if ($u_role_id == 3)
							{
								// Data Admin
								$redirect = 'category';
							}
							else if ($u_role_id == 2)
							{
								// User Admin
								$redirect = 'users';
							}

							// update user admin or data admin last login visit to today since it was not processed in the first login
							$u = ORM::factory('Admin', $u_id);
							if ($u->loaded())
							{
								$u->last_visit = date('Y-m-d H:i:s', time());
								$u->save();
							}

							// For concurrent user login check (user admin, data admin), other roles are already checked in check method
							// check first if there's already a record in the sessions table for this user
							$qry = DB::select()
								->from('sessions')
								->where('user_id', '=', $u_id)
								->where('bootout', 'IS', NULL)
								->as_object()
								->execute();

							if ($qry->count())
							{
								// mark the user session for bootout if found
								$sql = $qry->current();
								$sess_id = $sql->session_id;
								DB::update('sessions')->set(array('bootout'=>'Y'))->where('session_id', '=', $sess_id)->execute();
							}

							// Save user id in session table for the new login
							$sql = DB::update('sessions')->set(array('user_id'=>$u_id))->where('session_id', '=', $session->id())->execute();
							// END concurrent user login check

							// For Activity of User Admin or Data Admin
							$log_action = "Log In";
							$log_description = "Logged in the system on ".date('Y-m-d H:i:s', time())."\r\n\r\nChecked By: ".$user->name;
							Activity::add($u_id, $log_action, $log_description);

							// For activity of User Admin Checker or Data Admin Checker
							$log_action = "Log In";
							$log_description = "Logged in the system on ".date('Y-m-d H:i:s', time());
							Activity::add($user->id, $log_action, $log_description);

							$return = TRUE;
							$msg = 'Login Sucessful';
						}
						else
						{
							// correct username but wrong password
							$count = $login_try + 1;
							$sql = ORM::factory('Admin', $user_id);
							$sql->login_try = $count;
							$sql->save();

							// 3 consecutive wrong password attempt.  lock account
							$wrong_pass_count = $session->get('wrong_pass_count') ? $session->get('wrong_pass_count') : 0;
							if (($wrong_pass_count + 1) >= 3)
							{
								$this->lock_account($user_id);

								$msg = 'Your password has been locked due to '.$int_login_try.' invalid login attempts';
								$msg .= '<br />Please contact Level 1 User to re-activate your password';

								// For Activity
								$log_action = "Log In";
								$log_description = "Account is locked due to ".$int_login_try." invalid login attempts";

								// Log Activity
								Activity::add($user_id, $log_action, $log_description);
							}
							else
							{
								$c = $wrong_pass_count + 1;
								$session->set('wrong_pass_count', $c);
								$session->set('login_err', 'Y');

								$msg = 'Username and Password do not match.  Please try again.';

								// For Activity
								$log_action = "Log In";
								$log_description = "Username and Password do not match. (Username exists but did not match the password)";

								// Log Activity
								Activity::add($user_id, $log_action, $log_description);
							}

							Kohana::$log->add(Log::ERROR, 'Unable to find username/password match, username found in our DB');

							$return = FALSE;
						}
					}
				} else {
					$msg = 'Username and Password do not match.  Please try again';
					Kohana::$log->add(Log::ERROR, 'Username and Password do not match or user role is not User/Data Admin Checker');
				}
			}
			else
			{
				Kohana::$log->add(Log::ERROR, 'CSRF detected');
			}
		}
		else
		{
			Kohana::$log->add(Log::ERROR, 'Form method is not POST - User/Data Admin Checker');
		}

		if ($return == TRUE)
		{
			$this->redirect('admin/'.$redirect);
		}
		else
		{
			$session->set('msg', $msg);
			$this->redirect('admin/login/confirm');
		}
	}
}