<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Promo extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	public function action_index()
	{
		$nav = 'nav_promo';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$flash_data = $this->session->get_once('flash_data');

		$sql = ORM::factory('PromoCategory')
			->order_by('sort')
			->find_all();


		$view = View::factory('admin/promo/index');
		$view->navlink = $this->navlink();
		$view->sql = $sql;
		$view->flash_data = $flash_data;

		$this->template->content = $view;
	}

	private function navlink()
	{
		$link = View::factory('admin/promo/navlink');

		return $link;
	}

	public function action_create()
	{
		$data = array();
		$view = View::factory('admin/promo/form');
		$view->navlink = $this->navlink();
		$view->data = $data;
		$view->request_action = $this->request->action();

		$this->template->content = $view;
	}

	public function action_save()
	{
		$this->auto_render = FALSE;

		$error = TRUE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$name = Arr::get($this->request->post(), 'name');
				$id = Arr::get($this->request->post(), 'id');
				$checker_name = $this->session->get('checker_name');
				if ($id)
				{
					$sql = ORM::factory('PromoCategory', $id);
					$old_name = html_entity_decode($sql->name);
					$log_description = 'Update Promo Category from "'.$old_name.'" to "'.$name.'"';
					$action = 'update';
				}
				else
				{
					$sql = ORM::factory('PromoCategory');
					$log_description = 'Add Promo Category "'.$name.'"';
					$action = 'create';
				}
				$log_description .= "\r\nChecked By: ".$checker_name;

				$sql->name = $name;

				if( ! empty($_FILES))
				{
					$files = Validation::factory($_FILES)
						->rules('image', array(
					      array('Upload::not_empty'),
					      array('Upload::valid'),
					      array('Upload::type', array(':value', array('gif', 'jpg', 'jpeg', 'png'))),
					      array('Upload::size', array(':value', '5M'))
				   ));

					if ($files->check())
					{
						// Template File
						$directory = DOCROOT.'media/uploads';

						$filename  = preg_replace('/\s+/u', '_', $files['image']['name']);
						$file_name = uniqid().'_'.$filename;
						// Upload!
						Upload::save($files['image'], $file_name, $directory);

						$sql->image = $file_name;
					}
				}

				$sql->save();

				$error = FALSE;

				$log_action = 'Promo Category';
				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->session->set('flash_msg', $error);
				$this->session->set('flash_action', $action);

				$this->redirect('admin/promo/msg_view');
			}
		}
	}

	public function action_msg_view()
	{
		$view = View::factory('admin/promo/msg_view');
		$view->navlink = $this->navlink();
		$view->flash_msg = $this->session->get_once('flash_msg');
		$view->flash_action = $this->session->get_once('flash_action');

		$this->template->content = $view;
	}

	public function action_edit()
	{
		$id = $this->request->param('id');

		$sql = ORM::factory('PromoCategory', $id);
		$data = array(
			'id' => $sql->id,
			'name' => $sql->name,
			'image' => $sql->image
		);

		$view = View::factory('admin/promo/form');
		$view->navlink = $this->navlink();
		$view->data = $data;
		$view->request_action = $this->request->action();

		$this->template->content = $view;
	}

	public function action_remove_file()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$page_id = Arr::get($this->request->post(), 'page_id');
			$field = Arr::get($this->request->post(), 'field');
			$file = Arr::get($this->request->post(), 'file');

			$docroot = DOCROOT.'media/uploads';

			if (is_file($docroot.'/'.$file) AND file_exists($docroot.'/'.$file))
			{
				@unlink($docroot.'/'.$file);

				$sql = ORM::factory('PromoCategory', $page_id);
				if ($sql->loaded())
				{
					$sql->{$field} = '';

					$sql->save();
				}

				echo json_encode(true);
				exit;
			}

			echo json_encode(false);
		}
	}

	public function action_show_comment_form()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$view = View::factory('admin/promo/comment_form');

			$this->response->body($view);
		}
	}

	public function action_check_cms()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$data = FALSE;
			$promo_category_id = Arr::get($this->request->post(), 'promo_category_id');

			// check CMS if there's an existing page that has this content type
			$sql = ORM::factory('Page')
				->where('content_type_id', '=', 4)
				->where('product_category', '=', $promo_category_id)
				->find_all();

			if ($sql->count())
			{
				$data = TRUE;
			}
			echo json_encode($data);
			exit;
		}

		throw HTTP_Exception::factory(500, 'Access not allowed.');
	}

	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');

				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();
				$error = TRUE;

				$sql = ORM::factory('PromoCategory', $id);
				if ($sql->loaded())
				{
					// $sql->deleted = 'Y';
					$name = html_entity_decode($sql->name);
					$sql->delete();

					$log_action = "Promo Category";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Delete \"".$name."\" Promo Category\r\nChecked By: ".$checker_name."\r\n\r\n".$comment;

					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$error = FALSE;
					$action = 'delete';
					$this->session->set('flash_msg', $error);
					$this->session->set('flash_action', $action);

					$this->redirect('admin/promo/msg_view');
				}
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	public function action_header_image()
	{
		$sql = DB::select()
			->from('promo_header_img')
			->where('id', '=', 1)
			->as_object()
			->execute();

		if ($sql->count())
		{
			$res = $sql->current();
			$data = array(
				'id' => $res->id,
				'image' => $res->image
			);
		} else {
			$data = array();
		}

		$flash_data = $this->session->get_once('flash_data');

		$view = View::factory('admin/promo/header_image');
		$view->navlink = $this->navlink();
		$view->flash_data = $flash_data;
		$view->data = $data;

		$this->template->content = $view;
	}

	public function action_save_header_image()
	{
		$this->auto_render = FALSE;

		$error = TRUE;
		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$id = Arr::get($this->request->post(), 'id');

				// check existing image
				$image_hid = Arr::get($this->request->post(), 'image_hid');

				if( ! empty($_FILES))
				{
					$files = Validation::factory($_FILES)
						->rules('image', array(
					      array('Upload::not_empty'),
					      array('Upload::valid'),
					      array('Upload::type', array(':value', array('gif', 'jpg', 'jpeg', 'png'))),
					      array('Upload::size', array(':value', '5M'))
				   ));

					if ($files->check())
					{
						// Template File
						$directory = DOCROOT.'media/uploads';

						$filename  = preg_replace('/\s+/u', '_', $files['image']['name']);
						$file_name = uniqid().'_'.$filename;
						// Upload!
						Upload::save($files['image'], $file_name, $directory);

						$sql = DB::select()
							->from('promo_header_img')
							->where('id', '=', $id)
							->as_object()
							->execute();
						if ($sql->count())
						{
							$qry = DB::update('promo_header_img')
								->set(array('image'=>$file_name))
								->where('id', '=', $id)
								->execute();
						} else {
							$qry = DB::insert('promo_header_img', array('image'))->values(array($file_name))->execute();
						}

						$log_action = "Promo Category Header Image";
						$checker_name = $this->session->get('checker_name');
						$log_description = "Updated Promo Category Header Image\r\nChecked By: ".$checker_name;

						Activity::add($this->session->get('user_id'), $log_action, $log_description);

						$this->session->set('flash_data', 'The changes have been saved.');
					}
				}

				$this->redirect('admin/promo/header_image');
			}
		}
	}
}