<?php defined('SYSPATH') or die('No direct script access.');

class Email {

	public static function send(array $user_role, $subject, $category_id, $page_id, $extra=0)
	{
		$result = 1;

		if (is_array($user_role))
		{
			SwiftMailer::init();
			// Create the Transport

			$smtp = Kohana::$config->load('site')->get('smtp');

			try
			{
				if ($smtp)
				{
					// '192.168.0.7'
					$transport = Swift_SmtpTransport::newInstance($smtp, 25);
				}
				else
				{
					$transport = Swift_MailTransport::newInstance();
				}

				$mailer = Swift_Mailer::newInstance($transport);

				$logger = new Swift_Plugins_Loggers_EchoLogger();
				$mailer->registerPlugin(new Swift_Plugins_LoggerPlugin($logger));

				// Message instance with Subject
				$message = Swift_Message::newInstance($subject);

				// Set a From: address including a name
				$message->setFrom(array('cms@bpi.com.ph' => 'BPI Express Online CMS'));

				// get Page Information
				$page = ORM::factory('Page', $page_id);
				$control_no = $page->control_no;
				$title = html_entity_decode($page->title);
				$content_type = Arr::get(MyHelper::content_types(), $page->content_type_id);

				$history_list = '';
				$history = ORM::factory('DocumentHistory')
					->where('page_id', '=', $page_id)
					->order_by('date_added', 'DESC')
					->find_all();
				foreach ($history as $dh)
				{
					$h = Date('m/d/Y h:i a', strtotime($dh->date_added)).' '.$dh->user_name.' '.$dh->role.' '.$dh->description;
					$history_list .= ($history_list ? '<br />' : '').$h;
				}

				$body = "<table>
					<tr><td style='width:150px;'>Control Number:</td><td>".$control_no."</td>
					<tr><td>Title:</td><td>".$title."</td>
					<tr><td>Content Type:</td><td>".$content_type."</td>
					<tr><td>Document History:</td><td>".$history_list."</td>
				</table>";

				$user_roles = implode(",", $user_role);
				$sql = ORM::factory('Admin')
					->where('role_id', 'IN', DB::expr('('.$user_roles.')'))
					->where(DB::expr('\',\'+').'category_ids'.DB::expr('+\',\''), 'LIKE', "%,$category_id,%")
					->where('active', '=', 'Y')
					->find_all();

				if ($sql->count())
				{
					$recipients = array();
					foreach ($sql as $res)
					{
						// $email = Swift_Validate::email($res->email);
						$email = $res->email;
						// $message->addTo($email, $res->name);
						$recipients[$email] = $res->name;
					}

					if ($extra == 1)
					{
						// $message->addTo('BPIExpressonlineMails@bpi.com.ph', 'BPI Express Online Mails');
						$recipients['BPIExpressonlineMails@bpi.com.ph'] = 'BPI Express Online Mails';
					}

					$message->setTo($recipients);
					$message->setBody($body, 'text/html');

					// Send the message
					if($mailer->send($message))
					{
						$result = 1;
					}
					else
					{
						$result = -1;
						$err_list = '';
						foreach ($err as $k => $e)
						{
							$err_list .= ($err_list ? "\r\n" : '').$e;
						}
						// Kohana::$log->add(Log::ERROR, $subject."\r\nError sending email to the following email address\r\n".$err_list);
						Kohana::$log->add(Log::ERROR, $logger->dump());
					}
				}
			}
			catch (Swift_TransportException $STe)
			{
				$result = -1;
				Kohana::$log->add(Log::ERROR, $STe->getMessage());
			}
			catch (Exception $e)
			{
				$result = -1;
				Kohana::$log->add(Log::ERROR, $e->getMessage());
			}
		}

		return $result;
	}

	public static function reminder(array $user_role, $category_id, $subject, $body, $extra=0)
	{
		$result = 1;

		if (is_array($user_role))
		{
			SwiftMailer::init();
			// Create the Transport

			$smtp = Kohana::$config->load('site')->get('smtp');

			try
			{
				if ($smtp)
				{
					// '192.168.0.7'
					$transport = Swift_SmtpTransport::newInstance($smtp, 25);
				}
				else
				{
					$transport = Swift_MailTransport::newInstance();
				}

				$mailer = Swift_Mailer::newInstance($transport);

				$logger = new Swift_Plugins_Loggers_EchoLogger();
				$mailer->registerPlugin(new Swift_Plugins_LoggerPlugin($logger));

				// Message instance with Subject
				$message = Swift_Message::newInstance($subject);

				// Set a From: address including a name
				$message->setFrom(array('cms@bpi.com.ph' => 'BPI Express Online CMS'));

				$user_roles = implode(",", $user_role);
				$sql = ORM::factory('Admin')
					->where('role_id', 'IN', DB::expr('('.$user_roles.')'))
					->where(DB::expr('\',\'+').'category_ids'.DB::expr('+\',\''), 'LIKE', "%,$category_id,%")
					->where('active', '=', 'Y')
					->find_all();

				if ($sql->count())
				{
					$recipients = array();
					foreach ($sql as $res)
					{
						// $email = Swift_Validate::email($res->email);
						$email = $res->email;
						// $message->addTo($email, $res->name);
						$recipients[$email] = $res->name;
					}

					if ($extra == 1)
					{
						// $message->addTo('BPIExpressonlineMails@bpi.com.ph', 'BPI Express Online Mails');
						$recipients['BPIExpressonlineMails@bpi.com.ph'] = 'BPI Express Online Mails';
					}

					$message->setTo($recipients);
					$message->setBody($body, 'text/html');

					// Send the message
					if($mailer->send($message))
					{
						$result = 1;
					}
					else
					{
						$result = -1;
						$err_list = '';
						foreach ($err as $k => $e)
						{
							$err_list .= ($err_list ? "\r\n" : '').$e;
						}
						// Kohana::$log->add(Log::ERROR, $subject."\r\nError sending email to the following email address\r\n".$err_list);
						Kohana::$log->add(Log::ERROR, $logger->dump());
					}
				}
			}
			catch (Swift_TransportException $STe)
			{
				$result = -1;
				Kohana::$log->add(Log::ERROR, $STe->getMessage());
			}
			catch (Exception $e)
			{
				$result = -1;
				Kohana::$log->add(Log::ERROR, $e->getMessage());
			}
		}

		return $result;
	}

	public static function contact_us($subject, $body, $from_email, $from_name)
	{
		SwiftMailer::init();
		// Create the Transport

		$smtp = Kohana::$config->load('site')->get('smtp');

		if ($smtp)
		{
			// '192.168.0.7'
			$transport = Swift_SmtpTransport::newInstance($smtp, 25);
		}
		else
		{
			$transport = Swift_MailTransport::newInstance();
		}

		$mailer = Swift_Mailer::newInstance($transport);

		// Message instance with Subject
		$message = Swift_Message::newInstance($subject);

		// Set a From: address including a name
		$message->setFrom(array($from_email => $from_name));

		// Set To
		// expressonline@bpi.com.ph
		$email_to = 'BPIExpressonlineMails@bpi.com.ph';
		$message->setTo(array($email_to => 'BPI Express Online Mails'));

		$message->setBody($body, 'text/html');

		// Send the message
		$result = $mailer->send($message);

		return $result;
	}

	public static function feedback($subject, $body, $from_email, $from_name, $form_type)
	{
		try
		{
			SwiftMailer::init();
			// Create the Transport

			$smtp = Kohana::$config->load('site')->get('smtp');

			if ($smtp)
			{
				// '192.168.0.7'
				$transport = Swift_SmtpTransport::newInstance($smtp, 25);
			}
			else
			{
				$transport = Swift_MailTransport::newInstance();
			}

			$mailer = Swift_Mailer::newInstance($transport);

			// Message instance with Subject
			$message = Swift_Message::newInstance($subject);

			// Set a From: address including a name
			$message->setFrom(array($from_email => $from_name));

			// Set To
			switch ($form_type)
			{
				case 'ayala_plans':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'corporate_email':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'cc_appointment':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'remittance':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'sustainability':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'annual_report':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'bpi_philam':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
				case 'auto_insurance':
					// $email_to = 'BPIExpressonlineMails@bpi.com.ph';
					// $email_to_name = 'Zhel';
					$recipients = array(
						'lvdeguzman@bpi.com.ph',
						'mdacantomayor@bpi.com.ph'
					);
					break;
			}

			// $message->setTo(array($email_to => $email_to_name));

			$message->setTo($recipients);
			$message->setBody($body, 'text/html');

			// Send the message
			if($mailer->send($message))
			{
				$result = 1;
			}
			else
			{
				$result = -1;
				$err_list = '';
				foreach ($err as $k => $e)
				{
					$err_list .= ($err_list ? "\r\n" : '').$e;
				}
				// Kohana::$log->add(Log::ERROR, $subject."\r\nError sending email to the following email address\r\n".$err_list);
				Kohana::$log->add(Log::ERROR, $logger->dump());
			}
		}
		catch (Swift_TransportException $STe)
		{
			$result = -1;
			Kohana::$log->add(Log::ERROR, $STe->getMessage());
		}
		catch (Exception $e)
		{
			$result = -1;
			Kohana::$log->add(Log::ERROR, $e->getMessage());
		}

		return $result;
	}
}