<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Categories extends Controller_Template {

	public $template = 'template/admin_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Level 1 and User Admin can access this module (id 1 and 2)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		$this->template->navmenu = View::factory('nav/main');
	}

	public function action_index()
	{
		$nav = 'nav_categories';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$sql = ORM::factory('Category')
				->where('parent_id', '=', 0)
				->order_by('sort')
				->find_all();

		$this->template->content = View::factory('admin/categories/index');
		$this->template->content->sql = $sql;
	}

	/**
	 * Create Main Category
	 */
	public function action_create()
	{
		$query = ORM::factory('Template')
				->find_all();
		$arr_templates = array();
		foreach ($query as $res)
		{
			$arr_templates[$res->id] = $res->name;
		}

		$data = array(
			'id' => 0,
			'parent_id' => 0,
			'level' => 'Main'
		);
		$this->template->content = View::factory('admin/categories/form');
		$this->template->content->data = $data;
		$this->template->content->arr_templates = $arr_templates;
	}

	public function action_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$name = HTML::chars(Arr::get($this->request->post(), 'name'));
			$parent_id = Arr::get($this->request->post(), 'parent_id');
			$level = Arr::get($this->request->post(), 'level');
			$template_id = Arr::get($this->request->post(), 'template_id');
			$id = Arr::get($this->request->post(), 'id');

			if ($id)
			{
				$sql = ORM::factory('Category', $id);
			}
			else
			{
				$sql = ORM::factory('Category');
			}

			$sql->parent_id = $parent_id;
			$sql->name = $name;
			$sql->template_id = $template_id;
			$sql->save();

			if ($level == 'Main')
			{
				$this->redirect('admin/categories');
			}
			else
			{
				$this->redirect('admin/categories/sub/'.$parent_id.'/'.$level);
			}
		}
	}

	public function action_edit()
	{
		$id = $this->request->param('id');

		$sql = ORM::factory('Category', $id);

		$data = array(
			'id' => $sql->id,
			'parent_id' => $sql->parent_id,
			'name' => $sql->name,
			'template_id' => $sql->template_id,
			'level' => 'Main'
		);

		$query = ORM::factory('Template')
				->find_all();
		$arr_templates = array();
		foreach ($query as $res)
		{
			$arr_templates[$res->id] = $res->name;
		}

		$this->template->content = View::factory('admin/categories/form');
		$this->template->content->data = $data;
		$this->template->content->arr_templates = $arr_templates;
	}

	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$result = FALSE;

			$id = Arr::get($this->request->post(), 'id');

			$sql = ORM::factory('Category', $id);
			if ($sql->loaded())
			{
				$category_model = new Model_Category;
				$category_model->recursivedelete($id);

				$result = TRUE;
			}

			echo json_encode($result);
		}
	}

	public function action_sub()
	{
		$parent_id = $this->request->param('id');
		$level = $this->request->param('id2');
		if ($level == 1)
		{
			$level_label = 'Level 1';
		}
		else if ($level == 2)
		{
			$level_label = 'Level 2';
		}
		else if ($level == 3)
		{
			$level_label = 'Level 3';
		}

		$cat = ORM::factory('Category', $parent_id);

		$sql = ORM::factory('Category')
				->where('parent_id', '=', $parent_id)
				->order_by('sort')
				->find_all();

		$this->template->content = View::factory('admin/categories/sub');
		$this->template->content->sql = $sql;
		$this->template->content->parent_id = $parent_id;
		$this->template->content->level = $level;
		$this->template->content->level_label = $level_label;
		$this->template->content->next_level = $level + 1;
		$this->template->content->cat = $cat;
	}

	public function action_create_sub()
	{
		$parent_id = $this->request->param('id');
		$level = $this->request->param('id2');
		if ($level == 1)
		{
			$level_label = 'Level 1';
		}
		else if ($level == 2)
		{
			$level_label = 'Level 2';
		}
		else if ($level == 3)
		{
			$level_label = 'Level 3';
		}

		$categ = ORM::factory('Category', $parent_id);

		$data = array(
			'id' => 0,
			'parent_id' => $parent_id
		);
		$this->template->content = View::factory('admin/categories/form_sub');
		$this->template->content->data = $data;
		$this->template->content->categ = $categ;
		$this->template->content->level = $level;
		$this->template->content->level_label = $level_label;
	}

	public function action_edit_sub()
	{
		$id = $this->request->param('id');
		$parent_id = $this->request->param('id2');

		$sql = ORM::factory('Category')
				->where('parent_id', '=', $parent_id)
				->order_by('sort')
				->find_all();

		$this->template->content = View::factory('admin/categories/sub');
		$this->template->content->sql = $sql;
		$this->template->content->parent_id = $parent_id;
	}

	public function action_delete_sub()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$result = FALSE;

			$id = Arr::get($this->request->post(), 'id');

			$sql = ORM::factory('Category', $id);
			if ($sql->loaded())
			{
				$category_model = new Model_Category;
				$category_model->recursivedelete($id);

				$result = TRUE;
			}

			echo json_encode($result);
		}
	}

	public function action_sort()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$data = Arr::get($this->request->post(), 'data');
			$section = Arr::get($this->request->post(), 'type');

			$arr_data = explode(",", $data);
			$sort = 1;
			foreach ($arr_data as $k => $v)
			{
				$d = explode("_", $v);
				$id = $d[1];

				$sql = ORM::factory('Category', $id);
				$sql->sort = $sort;
				$sql->save();

				$sort++;
			}
		}
	}
}