<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_ContentType extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');

		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	/**
	 * Content Type Blank Form (Add)
	 * @return [type] [description]
	 */
	public function action_index()
	{
		$nav = 'nav_ctype';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$view = View::factory('admin/content_type/index');
		$view->navlink = $this->navlink();

		$data = array();
		$view->main = View::factory('admin/content_type/form');
		$view->main->data = $data;

		$this->template->content = $view;
	}

	/**
	 * Edit Content Type
	 * @return [type] [description]
	 */
	public function action_edit()
	{
		$id = $this->request->param('id');

		$sql = ORM::factory('ContentType', $id);

		$view = View::factory('admin/content_type/index');
		$view->navlink = $this->navlink();

		$data = array(
			'id' => $sql->id,
			'name' => $sql->name,
			'charlimit' => $sql->charlimit,
			'def_tpl' => $sql->def_tpl
		);

		$view->main = View::factory('admin/content_type/form');
		$view->main->data = $data;

		$this->template->content = $view;
	}

	/**
	 * Display Created Content Type in the left side menu
	 * @return [type] [description]
	 */
	private function navlink()
	{
		$sql = ORM::factory('ContentType')
			->order_by('id')
			->find_all();

		$link = View::factory('admin/content_type/navlink');
		$link->header = 'Content Type';
		$link->sql = $sql;

		return $link;
	}

	/**
	 * Saves or Update a Content Type
	 * @return [type] [description]
	 */
	public function action_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$name = Arr::get($this->request->post(), 'name');
				$charlimit = Arr::get($this->request->post(), 'charlimit');
				$def_tpl = Arr::get($this->request->post(), 'def_tpl');
				$id = Arr::get($this->request->post(), 'id');

				if ($id)
				{
					$sql = ORM::factory('ContentType', $id);
					$qry = clone $sql;
					$action = "Edit";
					$name = $sql->name;

					$other_info = "Old Value\r\nCharacter Limit: ".$qry->charlimit."\r\nNo. of Templates Assigned: ".count(explode(",", $qry->def_tpl));
				}
				else
				{
					$sql = ORM::factory('ContentType');
					$sql->name = $name;
					$action = "Add";

					$other_info = '';
				}

				$sql->def_tpl = $def_tpl;
				$sql->charlimit = $charlimit;
				$sql->save();
				$ctype_id = $sql->id;

				$other_info .= ($other_info ? "\r\n\r\n" : "")."New Value\r\nCharacter Limit: ".$charlimit."\r\nNo. of Templates Assigned: ".count(explode(",", $def_tpl));

				// For Activity
				$log_action = "Content Type";
				$checker_name = $this->session->get('checker_name');
				$log_description = "{$action} \"{$name}\" Content Type\r\nChecked By: ".$checker_name."\r\n\r\n".$other_info;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => strtolower($action),
					'id' => $ctype_id
				);

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/contenttype/msg_view');
				exit;
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * List Available Templates that Admin can select/add for a Content Type
	 * @return [type] [description]
	 */
	public function action_select_tpl()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$selected_tpls = Arr::get($this->request->post(), 'selected_tpls');
			$content_type_id = Arr::get($this->request->post(), 'ctype_id');

			$sql = ORM::factory('Template')
				->where_open()
				->where('content_type_id', 'IS', NULL)
				->or_where('content_type_id', '=', 0)
				->where_close();
			if ($content_type_id)
			{
				$sql->or_where('content_type_id', '=', $content_type_id);
			}
			$tpls = $sql->find_all();

			$view = View::factory('admin/content_type/tpl_list');
			$view->tpls = $tpls;
			$view->arr_tpls = $selected_tpls ? explode(",", $selected_tpls) : array();
			// $view->btn_label = $content_type_id ? 'Assign' : 'Add';
			$view->btn_label = 'Assign';

			$this->response->body($view);
		}
	}

	/**
	 * Check if content type is already existing
	 */
	public function action_check()
	{
		$this->auto_render = FALSE;

		$result = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'name');

			$sql = ORM::factory('ContentType')
				->where('name', '=', $name)
				->find_all();
			$result = $sql->count();
		}

		if ($result)
		{
			echo json_encode(false);
		}
		else
		{
			echo json_encode(true);
		}
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/content_type/index');
		$view->navlink = $this->navlink();

		$view->main = View::factory('admin/content_type/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		if (Arr::get($arr_data, 'action') != 'delete')
		{
			$sql = ORM::factory('ContentType', Arr::get($arr_data, 'id'));
			$view->main->sql = $sql;
		}

		$view->main->arr_data = $arr_data;
		$this->template->content = $view;
	}

	/**
	 * Before deleting a content type, must check if there's an active page/cms assigned to it.
	 * Prevent deletion if found.
	 * @return [type] [description]
	 */
	public function action_check_cms()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$data = FALSE;
			$ctype_id = Arr::get($this->request->post(), 'ctype_id');

			// check CMS if there's an existing page that has this content type
			$sql = ORM::factory('Page')
				->where('content_type_id', '=', $ctype_id)
				->find_all();

			if ($sql->count())
			{
				$data = TRUE;
			}
			echo json_encode($data);
			exit;
		}

		throw HTTP_Exception::factory(500, 'Access not allowed.');
	}

	/**
	 * Delete a Content Type
	 * @return [type] [description]
	 */
	public function action_delete()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();
				$id = Arr::get($this->request->post(), 'id');

				$sql = ORM::factory('ContentType', $id);

				if ($sql->loaded())
				{
					$name = $sql->name;
					$sql->delete();

					// For Activity
					$log_action = 'Content Type';
					$checker_name = $this->session->get('checker_name');
					$log_description = "Delete \"{$name}\" Content Type\r\nChecked By: ".$checker_name."\r\n\r\n{$comment}";

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$data = array(
						'action' => 'delete',
						'id' => $id
					);

					$this->session->set('flash_data', json_encode($data));

					$this->redirect('admin/contenttype/msg_view');
				}
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}
}