<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Menu extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	public function action_index()
	{
		$nav = 'nav_menu';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$arr_segments = MyHelper::segments();
		$view = View::factory('admin/menu/index');
		$view->arr_segments = $arr_segments;

		$this->template->content = $view;
	}

	/**
	 * Display Menu List
	 * @return [type] [description]
	 */
	public function action_get_menus()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$segment_id = Arr::get($this->request->post(), 'segment_id');
			$segment_name = Arr::get($this->request->post(), 'segment_name');

			$sql = ORM::factory('Menu')
				->where('segment_id', '=', $segment_id)
				->order_by('sort')
				->find_all();

			$view = View::factory('admin/menu/menu_list');
			$view->sql = $sql;
			$view->segment_id = $segment_id;
			$view->segment_name = $segment_name;

			$this->response->body($view);
		}
	}

	/**
	 * Saves / Update Menu
	 * @return [type] [description]
	 */
	public function action_save_menu()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$name = Arr::get($this->request->post(), 'menu');
				$segment_id = Arr::get($this->request->post(), 'segment_id');

				$sql = ORM::factory('Menu');
				$sql->action = $name;
				$sql->segment_id = $segment_id;
				$sql->save();
				$menu_id = $sql->id;

				$action = "Add";

				// For Activity
				$segment = Arr::get(MyHelper::segments(), $segment_id);
				$log_action = "Menu";
				$checker_name = $this->session->get('checker_name');
				$log_description = "{$action} \"{$name}\" Menu for \"{$segment}\" Segment\r\nChecked By: ".$checker_name;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => strtolower($action),
					'id' => $menu_id
				);

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/menu/msg_view');
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * Check if Action is already existing
	 * @return [type] [description]
	 */
	public function action_check()
	{
		$this->auto_render = FALSE;

		$result = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'name');

			$sql = ORM::factory('Menu')
				->where('action', '=', $name)
				->find_all();
			$result = $sql->count();
		}

		if ($result)
		{
			echo json_encode(false);
		}
		else
		{
			echo json_encode(true);
		}
	}

	/**
	 * Display Message
	 * @return [type] [description]
	 */
	public function action_msg_view()
	{
		$view = View::factory('admin/menu/msg_view');

		$flash_data = $this->session->get_once('flash_data');
		$arr_data = json_decode($flash_data, true);

		if (Arr::get($arr_data, 'action') != 'delete')
		{
			$sql = ORM::factory('Menu', Arr::get($arr_data, 'id'));
			$view->sql = $sql;
		}

		$view->arr_data = $arr_data;
		$this->template->content = $view;
	}

	/**
	 * Display Categories that can be added to a menu
	 * @return [type] [description]
	 */
	public function action_get_categories()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$menu_id = Arr::get($this->request->post(), 'menu_id');

			$qry = ORM::factory('Menu', $menu_id);
			$arr_categories = $qry->categories ? explode(",", $qry->categories) : array();

			$sql = ORM::factory('Category')
				->order_by('name')
				->order_by('id')
				->find_all();

			$view = View::factory('admin/menu/category_list');
			$view->sql = $sql;
			$view->menu_id = $menu_id;
			$view->arr_categories = $arr_categories;

			$this->response->body($view);
		}
	}

	/**
	 * Update Selected Categories
	 * @return [type] [description]
	 */
	public function action_save_categories()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$menu_id = Arr::get($this->request->post(), 'menu_id');
				$categories = implode(",", Arr::get($this->request->post(), 'category'));

				$sql = ORM::factory('Menu', $menu_id);
				$name = $sql->action;
				$segment_id = $sql->segment_id;
				$old_categories = $sql->categories;
				$sql->categories = $categories;
				$sql->save();

				// For Activity
				$old_categs = '';
				$new_categs = '';
				if ($old_categories)
				{
					$qry = ORM::factory('Category')
						->where('id', 'IN', DB::expr('('.$old_categories.')'))
						->find_all();
					foreach ($qry as $res)
					{
						$old_categs .= ($old_categs ? ', ' : '').html_entity_decode($res->name);
					}
					unset($qry);
				}
				$qry = ORM::factory('Category')
					->where('id', 'IN', DB::expr('('.$categories.')'))
					->find_all();
				foreach ($qry as $res)
				{
					$new_categs .= ($new_categs ? ', ' : '').html_entity_decode($res->name);
				}
				unset($qry);

				$segment = Arr::get(MyHelper::segments(), $segment_id);
				$log_action = "Menu Category";
				$checker_name = $this->session->get('checker_name');
				$log_description = "Update \"{$segment} > {$name}\" Categories\r\nChecked By: ".$checker_name;
				if ($old_categs)
				{
					$log_description .= "\r\n\r\nOld Categories\r\n".$old_categs;
				}
				$log_description .= "\r\n\r\nNew Categories\r\n".$new_categs;

				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => 'update',
					'id' => $menu_id
				);

				// delete Navigation Menu Fragment Cache
				Fragment::delete('menu1');
				Fragment::delete('menu4');
				Fragment::delete('menu5');

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/menu/msg_view');
			}

			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		throw HTTP_Exception::factory(404, 'URL not found!');
	}

	/**
	 * Sort Menu
	 * @return [type] [description]
	 */
	public function action_sort()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$data = Arr::get($this->request->post(), 'data');
			$section = Arr::get($this->request->post(), 'type');

			$arr_data = explode(",", $data);
			$sort = 1;
			foreach ($arr_data as $k => $v)
			{
				$d = explode("_", $v);
				$id = $d[1];

				$sql = ORM::factory('Menu', $id);
				$sql->sort = $sort;
				$sql->save();
				unset($sql);

				$sort++;
			}
		}
	}

	public function action_update_menu()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$menu_id = Arr::get($this->request->post(), 'menu_id');
				$action = HTML::chars(Arr::get($this->request->post(), 'menu'));
				$segment_id = Arr::get($this->request->post(), 'segment_id');

				DB::update('menus')
					->set(array('action'=>$action))
					->where('id', '=', $menu_id)
					->where('segment_id', '=', $segment_id)
					->execute();

				// For Activity
				$segment = Arr::get(MyHelper::segments(), $segment_id);
				$log_action = "Menu";
				$checker_name = $this->session->get('checker_name');
				$log_description = "Update \"{$action}\" Menu for \"{$segment}\" Segment\r\nChecked By: ".$checker_name;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$data = array(
					'action' => 'update-menu',
					'id' => $menu_id
				);

				$this->session->set('flash_data', json_encode($data));

				$this->redirect('admin/menu/msg_view');
			}
		}
	}

	public function action_remove()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$token = Arr::get($this->request->post(), 'token');
			if (Security::check($token))
			{
				$menu_id = Arr::get($this->request->post(), 'menu_id');
				$segment_id = Arr::get($this->request->post(), 'segment_id');

				$sql = ORM::factory('Menu')
					->where('id', '=', $menu_id)
					->where('segment_id', '=', $segment_id)
					->find();

				if ($sql->id)
				{
					$menu = $sql->action;

					$sql->delete();
					unset($sql);

					// For Activity
					$segment = Arr::get(MyHelper::segments(), $segment_id);
					$log_action = "Menu";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Deleted \"{$menu}\" Menu from \"{$segment}\" Segment\r\nChecked By: ".$checker_name;

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$data = array(
						'action' => 'delete',
						'id' => $menu_id
					);

					$this->session->set('flash_data', json_encode($data));

					echo json_encode(TRUE);
				}
			}
		}
	}
}