<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Setting extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');
		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}

		$nav = 'nav_setting';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}
	}

	public function action_session()
	{
		$sql = ORM::factory('Setting', 1);

		$sess_time_warn = $sql->sess_time_warn;
		$sess_time_logout = $sql->sess_time_logout;

		$flash_data = $this->session->get_once('flash_data');

		$this->template->content = View::factory('admin/setting/session_form');
		$this->template->content->sess_time_warn = $sess_time_warn;
		$this->template->content->sess_time_logout = $sess_time_logout;
		$this->template->content->flash_data = $flash_data;
	}

	public function action_session_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$sess_time_warn = Arr::get($this->request->post(), 'sess_time_warn');
				$sess_time_logout = Arr::get($this->request->post(), 'sess_time_logout');

				$old_sess_time_warn = Arr::get($this->request->post(), 'old_sess_time_warn');
				$old_sess_time_logout = Arr::get($this->request->post(), 'old_sess_time_logout');

				$sql = ORM::factory('Setting', 1);
				$sql->sess_time_warn = $sess_time_warn;
				$sql->sess_time_logout = $sess_time_logout;
				$sql->save();

				// For Activity
				$activity = "\r\n";
				if ($sess_time_warn != $old_sess_time_warn)
				{
					$activity .= "\r\nSession warning duration from ".$old_sess_time_warn." seconds to ".$sess_time_warn." seconds";
				}
				if ($sess_time_logout != $old_sess_time_logout)
				{
					$activity .= "\r\nSession timeout duration from ".$old_sess_time_logout." seconds to ".$sess_time_logout." seconds";
				}

				$log_action = "Manage Sessions";
				$checker_name = $this->session->get('checker_name');
				$log_description = "Update Session Warning and Timeout Setting\r\nChecked By: ".$checker_name.$activity;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->session->set('flash_data', 'The changes have been saved.');

				$this->redirect('admin/setting/session');
			}
		}
	}

	public function action_db()
	{
		$sql = ORM::factory('Setting', 1);

		$purge_draft = $sql->purge_draft ? $sql->purge_draft : 3;
		$purge_denied = $sql->purge_denied ? $sql->purge_denied : 6;
		$purge_archived = $sql->purge_archived ? $sql->purge_archived : 18;

		$flash_data = $this->session->get_once('flash_data');

		$arr_duration = array(
			3 => '3 Months',
			6 => '6 Months',
			9 => '9 Months',
			12 => '12 Months',
			15 => '15 Months',
			18 => '18 Months',
			21 => '21 Months',
			24 => '24 Months'
		);

		$this->template->content = View::factory('admin/setting/db_form');
		$this->template->content->purge_draft = $purge_draft;
		$this->template->content->purge_denied = $purge_denied;
		$this->template->content->purge_archived = $purge_archived;
		$this->template->content->arr_duration = $arr_duration;
		$this->template->content->flash_data = $flash_data;
	}

	public function action_db_save()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$purge_draft = Arr::get($this->request->post(), 'purge_draft');
				$purge_denied = Arr::get($this->request->post(), 'purge_denied');
				$purge_archived = Arr::get($this->request->post(), 'purge_archived');

				$old_purge_draft = Arr::get($this->request->post(), 'old_purge_draft');
				$old_purge_denied = Arr::get($this->request->post(), 'old_purge_denied');
				$old_purge_archived = Arr::get($this->request->post(), 'old_purge_archived');

				$sql = ORM::factory('Setting', 1);
				$sql->purge_draft = $purge_draft;
				$sql->purge_denied = $purge_denied;
				$sql->purge_archived = $purge_archived;
				$sql->save();

				// For Activity
				$activity = "\r\n";
				if ($purge_draft != $old_purge_draft)
				{
					$activity .= "\r\nPurge Draft duration from ".$old_purge_draft." months to ".$purge_draft." months";
				}
				if ($purge_denied != $old_purge_denied)
				{
					$activity .= "\r\nPurge Denied duration from ".$old_purge_denied." months to ".$purge_denied." months";
				}
				if ($purge_archived != $old_purge_archived)
				{
					$activity .= "\r\nPurge Archived duration from ".$old_purge_archived." months to ".$purge_archived." months";
				}

				$log_action = "Manage Database";
				$checker_name = $this->session->get('checker_name');
				$log_description = "Update Purge Duration Setting\r\nChecked By: ".$checker_name.$activity;

				// Log Activity
				Activity::add($this->session->get('user_id'), $log_action, $log_description);

				$this->session->set('flash_data', 'The changes have been saved.');

				$this->redirect('admin/setting/db');
			}
		}
	}
}