<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Admin_Template extends Controller_Template {

	public $template = 'template/admin2_tpl';

	public function before()
	{
		parent::before();

		$this->session = Session::instance();

		//checking ifAdministrator is logged in
		if( ! $this->session->get('user_id'))
		{
			throw HTTP_Exception::factory(404, 'URL not found!');
		}

		$checklogin = MyHelper::check_user_login($this->session->get('user_id'));
		if ($checklogin == 'Y')
		{
			$this->redirect('admin/logout/push');
		}

		$user_role_id = $this->session->get('user_role_id');

		// Only Data Admin can access this module (id 3)
		if ($user_role_id != 3)
		{
			throw HTTP_Exception::factory(500, 'Access not allowed.');
		}
	}

	public function action_index()
	{
		$nav = 'nav_tpl';
		if ($this->session->get('nav_link') != $nav)
		{
			$this->session->set('nav_link', $nav);
		}

		$this->session->set('tpl_filter', 'all');

		$this->redirect('admin/template/list');
	}

	/**
	 * List All Templates or filtered by Content Type
	 * @return [type] [description]
	 */
	public function action_list()
	{
		// get list of Content Types
		$sql = ORM::factory('ContentType')
			->find_all();

		$arr_ctypes = array(
			'all' => 'All Templates',
			'unassigned' => 'Unassigned'
		);
		foreach ($sql as $res)
		{
			$arr_ctypes[$res->id] = $res->name;
		}

		// Get All Templates (with filtering)
		$sql = ORM::factory('Template');
		if ($this->session->get('tpl_filter') == 'unassigned')
		{
			$sql->where('content_type_id', '=', 0);
		} else if ($this->session->get('tpl_filter') > 0) {
			$sql->where('content_type_id', '=', $this->session->get('tpl_filter'));
		}
		$tpls = $sql->find_all();

		$flash_data = $this->session->get_once('flash_data');

		$view = View::factory('admin/template/index');
		$view->navlink = $this->navlink();

		$data = array();
		$view->data = $data;
		$view->arr_ctypes = $arr_ctypes;
		$view->tpls = $tpls;
		$view->filter = $this->session->get('tpl_filter');
		$view->flash_data = $flash_data;

		$this->template->content = $view;
	}

	private function navlink()
	{
		$link = View::factory('admin/template/navlink');

		return $link;
	}

	/**
	 * Set Search Filter Session
	 * @return [type] [description]
	 */
	public function action_filter()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$filter = Arr::get($this->request->post(), 'filter');

			$this->session->set('tpl_filter', $filter);

			$this->redirect('admin/template/list');
		}
	}

	/**
	 * Display Template Info
	 * @return [type] [description]
	 */
	public function action_info()
	{
		$tpl_id = $this->request->param('id');

		$tpl = ORM::factory('Template', $tpl_id);

		$view = View::factory('admin/template/info');
		$view->navlink = $this->navlink();
		$view->tpl = $tpl;

		$this->template->content = $view;
	}

	/**
	 * AJAX Update of Template Info ('character limit')
	 * @return [type] [description]
	 */
	public function action_update_tpl()
	{
		$this->auto_render = FALSE;

		$data = array('result' => false);
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$tpl_id = Arr::get($this->request->post(), 'tpl_id');
			$char_limit = Arr::get($this->request->post(), 'char_limit');

			$tpl = ORM::factory('Template', $tpl_id);
			$tpl->char_limit = $char_limit;
			$tpl->save();

			$data = array(
				'result' => true,
				'char_limit' => $char_limit,
				'char_limit2' => number_format($char_limit, 0)
			);
		}

		echo json_encode($data);
	}

	/**
	 * Upload a new Template
	 * @return [type] [description]
	 */
	public function action_upload()
	{
		$sql = ORM::factory('ContentType')
			->find_all();

		$arr_ctypes[] = '';
		foreach ($sql as $res)
		{
			$arr_ctypes[$res->id] = $res->name;
		}

		$view = View::factory('admin/template/upload');
		$view->navlink = $this->navlink();
		$view->arr_ctypes = $arr_ctypes;

		$this->template->content = $view;
	}

	public function action_save_upload()
	{
		$this->auto_render = FALSE;

		$error = TRUE;
		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$name = Arr::get($this->request->post(), 'name');
				$content_type_id = Arr::get($this->request->post(), 'content_type_id');
				$sections = Arr::get($this->request->post(), 'sections');
				$img_dimensions = Arr::get($this->request->post(), 'img_dimensions');
				$header_footer = Arr::get($this->request->post(), 'header_footer');

				$sql = ORM::factory('Template');
				$sql->name = $name;
				$sql->content_type_id = $content_type_id;
				$sql->sections = $sections;
				$sql->img_dimensions = $img_dimensions;
				$sql->header_footer = $header_footer;

				$content_types = MyHelper::content_types();
				$content_type = Arr::get($content_types, $content_type_id);

				$f1 = FALSE;
				$f2 = FALSE;
				if( ! empty($_FILES))
				{
					$files = Validation::factory($_FILES)
					    ->rules('template_file', array(
					        array('Upload::not_empty'),
					        array('Upload::valid'),
					        array('Upload::type', array(':value', array('php'))),
					        array('Upload::size', array(':value', '5M'))
				    	))
				    	->rules('thumbnail', array(
					        array('Upload::not_empty'),
					        array('Upload::valid'),
					        array('Upload::type', array(':value', array('gif', 'jpg', 'jpeg', 'png'))),
					        array('Upload::size', array(':value', '5M'))
				    	))
				    	->rules('main_img', array(
					        array('Upload::not_empty'),
					        array('Upload::valid'),
					        array('Upload::type', array(':value', array('gif', 'jpg', 'jpeg', 'png'))),
					        array('Upload::size', array(':value', '5M'))
				    	));

				   if ($files->check())
					{
						// Template File
						$directory = DOCROOT.'application/views/templates';
						$filename  = preg_replace('/\s+/u', '_', $files['template_file']['name']);
						$file_name = uniqid().'_'.$filename;
						// Upload!
						$f1 = Upload::save($files['template_file'], $file_name, $directory);
						$file_name = pathinfo($f1, PATHINFO_FILENAME);
						$sql->file_name = $file_name;
						// Thumbnail
						$tpl_dir = DOCROOT.'media/template';
						$filename  = preg_replace('/\s+/u', '_', $files['thumbnail']['name']);
						$thumbnail = uniqid().'_'.$filename;
						// Upload!
						$f2 = Upload::save($files['thumbnail'], $thumbnail, $tpl_dir);
						$sql->thumbnail = $thumbnail;
						// Main Image
						$filename  = preg_replace('/\s+/u', '_', $files['main_img']['name']);
						$main_img = uniqid().'_'.$filename;
						// Upload!
						$f3 = Upload::save($files['main_img'], $main_img, $tpl_dir);
						$sql->main_img = $main_img;

						// file upload is successful
						if ($f1 AND $f2 AND $f3)
						{
							$error = FALSE;
						}

						$info = "Template Name: ".$name."\r\nTemplate File: ".$files['template_file']['name'];
						$info .= "\r\nThumbnail: ".$files['thumbnail']['name']."\r\nMain Image: ".$files['main_img']['name'];
						$info .= "\r\nContent Type: ".$content_type."\r\nField Definition: ".$sections;
					}
				}


				if ( ! $error)
				{
					$sql->save();

					$log_action = 'Template';
					$checker_name = $this->session->get('checker_name');
					$log_description = "Uploaded a new template - \"{$name}\"\r\nChecked By: ".$checker_name."\r\n\r\nDetails\r\n".$info;
					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);
				}
			}
		}

		$this->session->set('flash_msg', $error);

		$this->redirect('admin/template/msg_view');
	}

	public function action_msg_view()
	{
		$view = View::factory('admin/template/msg_view');
		$view->navlink = $this->navlink();
		$view->flash_msg = $this->session->get_once('flash_msg');

		$this->template->content = $view;
	}

	/*
	public function action_preview()
	{
		$this->auto_render = FALSE;

		$tpl_id = $this->request->param('id');

		$tpl = ORM::factory('Template', $tpl_id);
	}
	*/

	public function action_view_main()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$main_img = Arr::get($this->request->post(), 'img');
			$view = '<div class="row"><div class="span9" style="text-align:center;"><img src="'.URL::site('media/template/'.$main_img).'"></div></div><div class="row"><div class="span9"></div>';

			$this->response->body($view);
		}
	}

	public function action_check_tpl()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$data = FALSE;
			$tpl_id = Arr::get($this->request->post(), 'tpl_id');

			// check CMS if there's an existing page that has this content type
			$sql = ORM::factory('Page')
				->where('template_id', '=', $tpl_id)
				->find_all();

			if ($sql->count())
			{
				$data = TRUE;
			}
			echo json_encode($data);
			exit;
		}

		throw HTTP_Exception::factory(500, 'Access not allowed.');
	}

	public function action_delete_tpl()
	{
		$this->auto_render = FALSE;

		if (HTTP_Request::POST == $this->request->method())
		{
			$token = Arr::get($this->request->post(), 'token');

			if (Security::check($token))
			{
				$data = FALSE;
				$tpl_id = Arr::get($this->request->post(), 'tpl_id');
				require_once Kohana::find_file('vendor', 'Html2Text/Html2Text');
				$comment = Arr::get($this->request->post(), 'comment');
				$html2text = new \Html2Text\Html2Text($comment);
				$comment = $html2text->get_text();

				$sql = ORM::factory('Template', $tpl_id);

				if ($sql->loaded())
				{
					$file_name = $sql->file_name;
					$thumbnail = $sql->thumbnail;
					$main_img = $sql->main_img;
					$tpl_name = $sql->name;

					$directory = DOCROOT.'application/views/templates';
					if (file_exists($directory.'/'.$file_name))
					{
						@unlink($directory.'/'.$file_name);
					}

					$tpl_dir = DOCROOT.'media/template';
					if (file_exists($tpl_dir.'/'.$thumbnail))
					{
						@unlink($tpl_dir.'/'.$thumbnail);
					}

					if (file_exists($tpl_dir.'/'.$main_img))
					{
						@unlink($tpl_dir.'/'.$main_img);
					}

					$sql->delete();

					// For Activity
					$log_action = "Template";
					$checker_name = $this->session->get('checker_name');
					$log_description = "Delete \"{$tpl_name}\" Template\r\nChecked By: ".$checker_name."\r\n\r\n".$comment;

					// Log Activity
					Activity::add($this->session->get('user_id'), $log_action, $log_description);

					$this->session->set('flash_data', 'Delete "'.$tpl_name.'" template is successful.');

					$this->redirect('admin/template');
				}
			}
		}

		throw HTTP_Exception::factory(500, 'Access not allowed.');
	}

	/**
	 * Check if template name is already existing
	 */
	public function action_check()
	{
		$this->auto_render = FALSE;

		$result = 0;
		if (HTTP_Request::POST == $this->request->method() AND $this->request->is_ajax())
		{
			$name = Arr::get($this->request->post(), 'name');

			$sql = ORM::factory('Template')
				->where('name', '=', $name)
				->find_all();
			$result = $sql->count();
		}

		if ($result)
		{
			echo json_encode(false);
		}
		else
		{
			echo json_encode(true);
		}
	}
}